const $ = (query) => {
    return document.getElementById(query) || document.querySelector(query)
};

const $$ = (query) => {
    return document.querySelectorAll(query)
};

const parseURL = (url) => {
    const parser = document.createElement('a');
    const searchObject = {};
    let queries, split, i;

    parser.href = url;

    queries = parser.search.replace(/^\?/, '').split('&');
    for (i = 0; i < queries.length; i++) {
        split = queries[i].split('=');
        searchObject[split[0]] = split[1];
    }

    return {
        protocol: parser.protocol,
        host: parser.host,
        hostname: parser.hostname,
        port: parser.port,
        pathname: parser.pathname,
        search: parser.search,
        searchObject: searchObject,
        hash: parser.hash,
        username: parser.username ?? '',
        password: parser.password ?? '',
        apiKey: parser.apiKey ?? ''
    };
};

const stripTrailingSlashes = (url) => {
    while (url.endsWith('/')) {
        url = url.slice(0, -1);
    }
    return url;
}

const fixDKSUrl = (url) => {
    url = stripTrailingSlashes(url);
    if (!url.endsWith("/api")) {
        url = url + "/api";
    }

    return url;
}

const request = async (method, url, credentials, onSuccess, onError, timeout) => {
  if (typeof(timeout) === 'undefined') {
    timeout = 0;
  }

  const options = {
    method: method,
    headers: {},
    credentials: 'omit'
  };

  if (credentials) {
    if (credentials.hasOwnProperty('apiKey') && credentials.apiKey !== '') {
        options.headers.Authorization = `Bearer ${credentials.apiKey}`;
    } else if (credentials.hasOwnProperty('username') && credentials.hasOwnProperty('password') && credentials.username !== '' && credentials.password !== '') {
        options.headers.Authorization = `Basic ${btoa(credentials.username + ":" + credentials.password)}`;
    }
  }

  let timeoutId = null;
  if (timeout > 0) {
    const controller = new AbortController();
    options.signal = controller.signal;
    timeoutId = setTimeout(() => {
        controller.abort();
    }, timeout);
  }

  try {
    const response = await fetch(url, options);
    if (timeoutId) clearTimeout(timeoutId); // Clear timeout if request succeeds

    if (response.ok) {
      const responseText = await response.text();
      onSuccess(responseText);
    } else {
      if (onError) {
        onError(response);
      }
    }
  } catch (e) {
    if (timeoutId) clearTimeout(timeoutId); // Clear timeout if request succeeds

    if (onError) {
      if (e.name === 'AbortError') {
        e.statusText = 'Timeout';
      } else {
        e.statusText = 'Invalid URL';
      }
      e.status = '0';
      onError(e);
    }
  }
};

const json_request = (method, url, credentials, onSuccess, onError, timeout) => {
    request(method, url, credentials, (value) => {
        try {
            const result = JSON.parse(value);
            onSuccess(result);
        } catch (error) {
            onError(error);
        }
    }, onError, timeout);
};

const selectByValue = (elementId, value) => {
    var options = $('#' + elementId).options;
    if (typeof (options) === 'undefined') return false;
    for (var i = 0, optionsLength = options.length; i < optionsLength; i++) {
        if (options[i].value === String(value)) {
            $('#' + elementId).selectedIndex = i;
            return true;
        }
    }
    return false;
}

class INI {
    static async load(uri) {
        return new Promise((resolve, reject) => {
            request('GET', uri, null, (value) => {
                resolve(this.parse(value));
            }, (error) => {
                console.error(error);
                resolve(null);
            })
        });
    }

    static parse(iniString) {
        const iniMap = {};
        const lines = iniString.split('\n');
        let currentSection = '';

        for (var i = 0; i < lines.length; ++i) {
            const line = lines[i].trim();
            if (line.startsWith(';') || line === '') {
                continue;
            } else if (line.startsWith('[')) {
                if (!line.endsWith(']')) {
                    console.error("Section must end with bracket in line " + i + ": " + line);
                    continue;
                }
                currentSection = line.substring(1, line.length - 1);
            } else if (line.indexOf('=') > -1) {
                const pos = line.indexOf('=');
                const key = line.substring(0, pos - 1).trim();
                const value = line.substring(pos + 1, line.length).trim();

                if ((value.startsWith('"') && value.endsWith('"')) || (value.startsWith("'") && value.endsWith("'"))) {
                    value = value.substring(1, value.length);
                }
                iniMap[currentSection === '' ? key : currentSection + '.' + key] = value.replace(/\\n/g, '\n').replace(/\\t/g, '\t');
            }
        }
        return iniMap;
    }
}

class Localization {
    static language = {};

    static async load(languageCode, fallbackLanguage) {
        languageCode = languageCode.toLowerCase();
        Localization.language = await INI.load(chrome.extension.getURL('assets/lang/' + languageCode + '.ini'));
        if (!Localization.language) {
            console.error("Could not load langauge: " + languageCode + ". Trying to load fallback language.");
            Localization.language = await INI.load(chrome.extension.getURL('assets/lang/' + fallbackLanguage + '.ini'));
            if (!Localization.language) {
                console.error("Could not load fallback language: " + fallbackLanguage);
            }
        }
    }

    static updateDocument(document) {
        var elements = document.querySelectorAll('[alt]');
        elements.forEach((element) => {
            element.alt = loca(element.alt);
        })

        elements = document.querySelectorAll('[data-lang]');
        elements.forEach((element) => {
            element.innerHTML = loca(element.attributes['data-lang'].value);
        })
    }
}

const loca = (locaKey) => {
    if (typeof (Localization.language[locaKey]) !== 'undefined') {
        return Localization.language[locaKey];
    }
    console.info("Couldn't find loca key: " + locaKey + " in language file.");
    return '{' + locaKey + '}';
}

String.prototype.assign = function (search, replace) {
    return this.split(search).join(replace);
}

const getRadioNodes = (name) => {
    var elements = document.querySelectorAll('input[name = "' + name + '"]');
    if (!elements) return [];
    return elements;
}

const getRadioVal = (name) => {
    let val = undefined;
    const radios = getRadioNodes(name);
    for (var i = 0, len = radios.length; i < len; i++) {
        if (radios[i].checked) {
            val = radios[i].value;
            break;
        }
    }

    return val;
}

const sendCommand = (command) => {
    chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
        if (tabs[0]) {
            chrome.tabs.sendMessage(
                tabs[0].id,
                { subject: command },
                (response) => { }
            );
        }
    });
}

const sendActivationMessage = (activate) => {
    chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
        if (tabs[0]) {
            chrome.tabs.sendMessage(
                tabs[0].id,
                { subject: 'activate', activate: activate },
                (response) => { }
            );
        }
    });
}

const setAutoscanState = (activate, tabId) => {
    chrome.tabs.sendMessage(
        tabId,
        { subject: 'autoscan', activate: activate },
        (response) => { }
    );
}

const getCurrentHostname = async () => {
    return new Promise((resolve) => {
        chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
            if (tabs[0]) {
                const tab = tabs[0];
                const urlInfo = parseURL(tab.url);
                return resolve(urlInfo.hostname);
            }
            return resolve(null);
        });
    });
};

const checkDKSVersion = async (apiEndpoint, credentials, timeout) => {
    return new Promise((resolve, reject) => {
        json_request('GET', apiEndpoint + '/version', credentials, (result) => {
            resolve(result);
        }, (error) => {
            reject(error);
        }, timeout);
    });
};

const buildUrlWithCredentials = (url, username, password) => {
    var urlInfo = parseURL(url);
    var credentials = '';
    if (username != '' && password != '') {
        credentials = username + ':' + password + '@';
    }

    return urlInfo.protocol + '//' + credentials + urlInfo.host + urlInfo.pathname;
}

const removeCredentialsFromUrl = (url) => {
    var urlInfo = parseURL(url);
    return urlInfo.protocol + '//' + urlInfo.host + urlInfo.pathname;
}